require 'textbubble/version'
require 'rainbow/ext/string'

module Textbubble
  module String
    ##
    # Extra methods for Strings implementing Textbubbles
    module InstanceMethods
      ##
      # Creates a bubble on the right hand side
      def rbubble(margin: 80, width: nil, fg: :black, bg: :white)
        lines = wrap_and_split(width.nil? ? (margin * 0.6).to_i : width)
        max = lines.map(&:length).max || 0
        output = ''
        pad = (margin - max - 3) > 0 ? margin - max - 3 : 0
        0.upto(lines.length - 2) do |i|
          output +=
            ' ' * pad + lines[i].pad(max).color(fg).background(bg) + "\n"
        end if lines.length > 1
        output += ' ' * pad + lines[-1].pad(max).color(fg).background(bg) +
                  '◣'.color(bg) + "\n"
      end

      ##
      # Creates a bubble on the left hand side
      def lbubble(width: 60, fg: :black, bg: :white)
        lines = wrap_and_split width
        max = lines.map(&:length).max || 0
        output = ''
        0.upto(lines.length - 2) do |i|
          output += ' ' + lines[i].pad(max).color(fg).background(bg) + "\n"
        end if lines.length > 1
        output += '◢'.color(bg) +
                  lines[-1].pad(max).color(fg).background(bg) + "\n"
      end

      ##
      # Add a space and left justify
      def pad(max_length)
        ' ' + ljust(max_length) + ' '
      end

      private

      def wrap(margin)
        margin = 40 if margin.nil? || margin < 2
        split("\n").collect do |line|
          if line.length > margin
            line.gsub(/(.{1,#{margin}})(\s+|$)/, "\\1\n").strip \
          else
            line
          end
        end * "\n"
      end

      def wrap_and_split(width)
        lines = wrap(width).split "\n"
        lines == [] ? [''] : lines
      end
    end
  end
end

::String.send(:include, Textbubble::String::InstanceMethods)
